#!/bin/sh

set -e
# set -xe

UNITTEST=1
. ./sdo-mount.sh

die()
{
    echo ""
    echo "ERROR: $1 ...  exiting"
    echo ""
    exit 1
}

test_passed()
{
    echo "PASSED"
}

check()
{
    fct=$1
    exp_ret=$2
    exp_output=$3
    ret=0
    # eval runs in a separate shell, it's environment
    # is not visible from this shell
    output=`eval $fct 2>&1` || ret=$?
    [ $ret -eq $exp_ret ] || die "$fct: return code mismatch [$exp_ret != $ret]"

    if [ -z "$exp_output" ]; then
        return
    fi

    [ "$output" = "$exp_output" ] || \
        die "$fct: output mismatch [\'$exp_output\' != \'$output\']"
}


echo "######################################"
echo "# TEST: isnum                        #"
echo "######################################"
check "isnum 0" 0
check "isnum 7 foo" 0
check "isnum " 1
check "isnum f" 1
check "isnum 3746477" 0
check "isnum -3746477" 0
check "isnum 3E+7" 1
test_passed


echo "######################################"
echo "# TEST: emit_log                     #"
echo "######################################"
check "emit_log check-ret-code" 0
FOO=0x7f
BAR=0xbeef
check "emit_log ipsum lirum $FOO $BAR" 0 "SD-CARD: ipsum lirum 0x7f 0xbeef"
test_passed

echo "
  DESTRUCTIVE TESTS

  The following tests will destroy the filesystem and partition
  table on the SD-card and verify sdo-mount can correctly restore them
"
echo -n "Continue? y/N "
read answer

if [ -z "$answer" -o "$answer" != 'y' ]; then
    echo "abort..."
    exit
fi

umount_card()
{
    if mountpoint -q /media/card; then
        umount /media/card
    fi
}

umount_card
if mountpoint -q /media/card; then
    die "run sdo-umount.sh before starting"
fi

echo > $REBOOT_LOG

echo "######################################"
echo "# TEST: test blkid                   #"
echo "######################################"
# might save us if we run on a laptop with a SD-card inserted
fetch_blkid_desc
check "test $LABEL = DSSDATA" 0
test_passed

echo "######################################"
echo "# TEST: destroy/restore filesystem   #"
echo "######################################"
destroy_filesystem()
{
    dd if=/dev/zero of=/dev/mmcblk0p1 bs=512 count=10 2> /dev/null
}

destroy_filesystem
mount_sd
check "mountpoint -q /media/card" 0
test_passed
umount_card

echo "######################################"
echo "# TEST: destroy fstab                #"
echo "######################################"
destroy_fstab()
{
    type partx >> /dev/null || \
        die "This test requires 'util-linux-partx', please install"

    # remove partition from kernel
    if [ -b /dev/mmcblk0p1 ]; then
        # TODO need util-linux-partx
        partx -d /dev/mmcblk0p1
    fi
    dd if=/dev/zero of=/dev/mmcblk0 bs=512 count=10 2> /dev/null
}

destroy_fstab
mount_sd
check "mountpoint -q /media/card" 0
test_passed
umount_card


echo "######################################"
echo "# TEST: remove journal               #"
echo "######################################"

# add journal
tune2fs -j /dev/mmcblk0p1 2> /dev/null
check "has_journal" 0

mount_sd # shall remove the journal
check "has_journal" 1

test_passed
umount_card


echo "
  ALL PASSED

  Don't forget to run sdo-mount from cmdline as well. The main()
  function in sdo-mount is not covered by this test. Read: we really
  had bugs in there in the past.

  ./sdo-umount.sh
  ./sdo-mounts.sh
"
