#!/bin/sh

TEST_DIR=/tmp/sdcard_tests

UNITTEST=1
. ./sdo-trim.sh

# overwrite global variable
CMDLINE=${TEST_DIR}/cmdline
FSTAB=${TEST_DIR}/fstab
MTAB=${TEST_DIR}/mtab
ROOTFS_MNTPOINT=${TEST_DIR}/rootfs

# only log what would be done
#DO='echo'

DIR1=${TEST_DIR}/dir1
DIR2=${TEST_DIR}/dir2
BACKUP_DIR=${TEST_DIR}/backup

clean_autofiles()
{
    rm -rf $TEST_DIR

    # assume these exist on every systems, fair assumption?
    # otherwise fix grep $MTAB calls
    mkdir ${TEST_DIR} 2> /dev/null
    echo > $MTAB
    echo > $CMDLINE
    echo > $FSTAB
}

create_test_files()
{
    clean_autofiles

    mkdir -p $DIR1/subdir
    touch $DIR1/foo
    touch $DIR1/subdir/bro
    touch $DIR1/subdir/bor
    touch $DIR1/bar

    mkdir -p $DIR2/subdir
    touch $DIR2/foo
    touch $DIR2/brom
    touch $DIR2/subdir/bro
    touch $DIR2/subdir/ber
}

create_test_files
NAMES=""
for i in `collisions $DIR1 $DIR2`; do
    NAMES="$NAMES $i"
done
if [ "x$NAMES" != "x foo subdir/bro" ]; then
    echo "BUG>>>> collisions: $NAMES"
fi

create_test_files
backup_file $DIR1/foo $BACKUP_DIR
EXPECT="$BACKUP_DIR/$(date "+%Y-%m-%d")/$DIR1/foo"
if [ ! -f $EXPECT ]; then
    echo "BUG>>>> backup_file: $(find $BACKUP_DIR -type f)"
fi

mockup_cmdline_valid() {
    clean_autofiles
    cat > $CMDLINE <<EOF
    console=ttyS0,115200 root=/dev/mtdblock4 rw rootfstype=jffs2
    EOF
}

mockup_mtab_rootfs_mounted() {
    mockup_cmdline_valid
    cat > $MTAB <<EOF
    /dev/mtdblock4 ${ROOTFS_MNTPOINT} jffs2 rw,relatime 0 0
    EOF
}

mockup_mtab_rootfs_mounted_wrong_mnt_point() {
    mockup_cmdline_valid
    cat > $MTAB <<EOF
    /dev/mtdblock4 ${ROOTFS_MNTPOINT}_nasty jffs2 rw,relatime 0 0
    EOF
}

mockup_mtab_rootfs_mounted_wrong_root_device() {
    mockup_cmdline_valid
    cat > $MTAB <<EOF
    /dev/wrong ${ROOTFS_MNTPOINT} jffs2 rw,relatime 0 0
    EOF
}

#
# check if rootfs already mounted detection works
#
mockup_mtab_rootfs_mounted
if ! rootfs_mounted; then
    echo "BUG>>> rootfs mounted not detected"
fi

mockup_mtab_rootfs_mounted_wrong_mnt_point
if rootfs_mounted; then
    echo "BUG>>> rootfs mounted: wrong mount point not detected"
fi

mockup_mtab_rootfs_mounted_wrong_root_device
if rootfs_mounted; then
    echo "BUG>>> rootfs mounted: wrong root device not detected"
fi

#
# check the whole mount_rootfs
#

# prevent execution
DO="echo -n"

mockup_cmdline_wrong_root_device() {
    clean_autofiles
    cat > $CMDLINE <<EOF
    console=ttyS0,115200 root=/dev/mtdflock4 rw rootfstype=jffs2
    EOF
}
mockup_cmdline_wrong_rootfs() {
    clean_autofiles
    cat > $CMDLINE <<EOF
    console=ttyS0,115200 root=/dev/mtdblock4 rw rootfstype=glibs
    EOF
}

mockup_cmdline_valid
CMD_LOG=$(mount_rootfs)
SHOULD="mkdir -p ${ROOTFS_MNTPOINT}mount -t jffs2 /dev/mtdblock4 ${ROOTFS_MNTPOINT}"
if [ "x$CMD_LOG" !=  "x$SHOULD" ]; then
    echo "BUG>>> valid rootfs fails: $CMD_LOG"
fi

mockup_cmdline_wrong_root_device
CMD_LOG=$(mount_rootfs 2>&1)
ERR_MSG=`expr "$CMD_LOG" : 'ERROR:.*\(root device\)'`
if [ "x$ERR_MSG" != "xroot device" ]; then
    echo "BUG>>> inalid root device succeeds: $CMD_LOG"
fi

mockup_cmdline_wrong_rootfs
CMD_LOG=$(mount_rootfs 2>&1)
ERR_MSG=`expr "$CMD_LOG" : 'ERROR:.*\(unsupported filesystem\)'`
if [ "x$ERR_MSG" != "xunsupported filesystem" ]; then
    echo "BUG>>> inalid rootfs succeeds: $CMD_LOG"
fi

#
# filter out overlay mounts from fstab
#
create_fstab()
{
    RO=$1
    RW=$2
    if [ -n "$3" -a "$3" = "functional" ]; then
        cat > $FSTAB <<EOF
        aufs   ${RO} aufs nodev,noatime,br:${RW}=rw:${RO}=ro    0    0
        EOF
        return
    fi

    cat > $FSTAB <<EOF
    aufs   ${RO}1 aufs nodev,noatime,br:${RW}1=rw:${RO}1=ro    0    0
    aufs   ${RO}2 aufs nodev,noatime,br:${RO}2=ro:${RW}2=rw    0    0
    aufs   ${RO}3 aufs nodev,noatime,br:${RW}3:${RO}3    1    0
    EOF
}

i=0
create_fstab $DIR1 $DIR2
parse_overlays | while read RO RW; do
i=$((i+1))
if [ "x$RO" != "x${DIR1}$i" -o "x$RW" != "x${DIR2}$i" ]; then
    echo "BUG$i>>> parse_overlays: ro:$RO rw:$RW"
fi
done
CT=$(parse_overlays | wc -l)
if [ $CT -ne 3 ]; then
    echo "BUG>>> parse_overlays parsed $CT line(s) instead of 3"
fi

#
# test the whole thing
#
DO=""
create_test_files
create_fstab $DIR1 $DIR2 functional
parse_overlays | while read RO RW; do
for REL_PATH in `collisions $RO $RW`; do
    echo "$RO> backing up $REL_PATH"
    backup_file $RO/$REL_PATH $BACKUP_DIR
done
done
